@@ -12,25 +12,32 @@
  * for more details.
  */
 
-#include "application.h"
 #include "owncloudgui.h"
-#include "theme.h"
-#include "folderman.h"
-#include "configfile.h"
-#include "progressdispatcher.h"
-#include "owncloudsetupwizard.h"
-#include "sharedialog.h"
-#include "settingsdialog.h"
-#include "logger.h"
-#include "logbrowser.h"
-#include "account.h"
-#include "accountstate.h"
-#include "openfilemanager.h"
-#include "accountmanager.h"
 #include "aboutdialog.h"
+#include "account.h"
+#include "accountmanager.h"
+#include "accountstate.h"
+#include "application.h"
 #include "common/syncjournalfilerecord.h"
+#include "configfile.h"
 #include "creds/abstractcredentials.h"
+#include "folderman.h"
+#include "folderwizard/folderwizard.h"
+#include "gui/accountsettings.h"
 #include "guiutility.h"
+#include "logbrowser.h"
+#include "logger.h"
+#include "openfilemanager.h"
+#include "progressdispatcher.h"
+#include "settingsdialog.h"
+#include "setupwizardcontroller.h"
+#include "sharedialog.h"
+#include "theme.h"
+
+#include "libsync/graphapi/space.h"
+#include "libsync/graphapi/spacesmanager.h"
+
+#include "resources/resources.h"
 
 #include <QDesktopServices>
 #include <QDir>
@@ -41,8 +48,64 @@
 
 #ifdef WITH_LIBCLOUDPROVIDERS
 #include "libcloudproviders/libcloudproviders.h"
+#include "selectivesyncdialog.h"
 #endif
 
+using namespace std::chrono_literals;
+
+namespace {
+
+using namespace OCC;
+
+void setUpInitialSyncFolder(AccountStatePtr accountStatePtr, bool useVfs)
+{
+    auto folderMan = FolderMan::instance();
+
+    // saves a bit of duplicate code
+    auto addFolder = [folderMan, accountStatePtr, useVfs](const QString &localFolder, const QString &remotePath, const QUrl &webDavUrl, const QString &displayName = {}) {
+        return folderMan->addFolderFromWizard(accountStatePtr, localFolder, remotePath, webDavUrl, displayName, useVfs);
+    };
+
+    auto finalize = [accountStatePtr] {
+        accountStatePtr->checkConnectivity();
+        FolderMan::instance()->setSyncEnabled(true);
+        FolderMan::instance()->scheduleAllFolders();
+    };
+
+    if (accountStatePtr->supportsSpaces()) {
+        // Qt6: Qt::SingleShotConnection
+        auto *singleShotConnection = new QObject();
+        QObject::connect(accountStatePtr->account()->spacesManager(), &GraphApi::SpacesManager::ready, singleShotConnection,
+            [accountStatePtr, addFolder, finalize, singleShotConnection] {
+                delete singleShotConnection;
+                auto spaces = accountStatePtr->account()->spacesManager()->spaces();
+                // we do not want to set up folder sync connections for disabled spaces (#10173)
+                spaces.erase(std::remove_if(spaces.begin(), spaces.end(), [](auto *space) { return space->disabled(); }), spaces.end());
+
+                if (!spaces.isEmpty()) {
+                    const QString localDir(accountStatePtr->account()->defaultSyncRoot());
+                    FileSystem::setFolderMinimumPermissions(localDir);
+                    Folder::prepareFolder(localDir);
+                    Utility::setupFavLink(localDir);
+                    for (const auto *space : spaces) {
+                        const QString name = space->displayName();
+                        const QString folderName = FolderMan::instance()->findGoodPathForNewSyncFolder(localDir, name);
+                        auto folder = addFolder(folderName, {}, QUrl(space->drive().getRoot().getWebDavUrl()), name);
+                        folder->setPriority(space->priority());
+                        // save the new priority
+                        folder->saveToSettings();
+                    }
+                    finalize();
+                }
+            });
+        accountStatePtr->account()->spacesManager()->checkReady();
+    } else {
+        addFolder(accountStatePtr->account()->defaultSyncRoot(), Theme::instance()->defaultServerFolder(), accountStatePtr->account()->davUrl());
+        finalize();
+    }
+}
+}
+
 namespace OCC {
 
 const char propertyAccountC[] = "oc_account";
@@ -55,7 +118,7 @@ ownCloudGui::ownCloudGui(Application *parent)
     , _app(parent)
 {
     // for the beginning, set the offline icon until the account was verified
-    _tray->setIcon(Theme::instance()->folderOfflineIcon(/*systray?*/ true, /*currently visible?*/ false));
+    _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Status::Offline, true, false));
 
     connect(_tray, &QSystemTrayIcon::activated,
         this, &ownCloudGui::slotTrayClicked);
@@ -146,13 +209,6 @@ void ownCloudGui::slotSyncStateChange(Folder *folder)
 
     qCInfo(lcApplication) << "Sync state changed for folder " << folder->remoteUrl().toString() << ": " << result.statusString();
 
-    if (result.status() == SyncResult::Success
-        || result.status() == SyncResult::Problem
-        || result.status() == SyncResult::SyncAbortRequested
-        || result.status() == SyncResult::Error) {
-        Logger::instance()->enterNextLogFile();
-    }
-
     if (result.status() == SyncResult::NotYetStarted) {
         _settingsDialog->slotRefreshActivity(folder->accountState());
     }
@@ -177,13 +233,13 @@ void ownCloudGui::slotAccountStateChanged()
 
 void ownCloudGui::slotTrayMessageIfServerUnsupported(Account *account)
 {
-    if (account->serverVersionUnsupported()) {
+    if (account->serverSupportLevel() != Account::ServerSupportLevel::Supported) {
         slotShowTrayMessage(
             tr("Unsupported Server Version"),
             tr("The server on account %1 runs an unsupported version %2. "
                "Using this client with unsupported server versions is untested and "
                "potentially dangerous. Proceed at your own risk.")
-                .arg(account->displayName(), account->serverVersion()));
+                .arg(account->displayName(), account->capabilities().status().versionString()));
     }
 }
 
@@ -211,7 +267,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
         }
     }
 
-    const auto &map = FolderMan::instance()->map();
+    const auto &map = FolderMan::instance()->folders();
     for (auto *f : map) {
         if (!f->syncPaused()) {
             allPaused = false;
@@ -219,7 +275,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     }
 
     if (!problemAccounts.empty()) {
-        _tray->setIcon(Theme::instance()->folderOfflineIcon(true, contextMenuVisible()));
+        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Status::Offline, true, contextMenuVisible()));
         if (allDisconnected) {
             setStatusText(tr("Disconnected"));
         } else {
@@ -236,10 +292,9 @@ void ownCloudGui::slotComputeOverallSyncStatus()
         QStringList messages;
         messages.append(tr("Disconnected from accounts:"));
         for (const auto &a : qAsConst(problemAccounts)) {
-            QString message = tr("Account %1: %2").arg(a->account()->displayName(), a->stateString(a->state()));
+            QString message = tr("Account %1").arg(a->account()->displayName());
             if (!a->connectionErrors().empty()) {
-                message += QLatin1String("\n");
-                message += a->connectionErrors().join(QLatin1String("\n"));
+                message += QLatin1String("\n") + a->connectionErrors().join(QLatin1String("\n"));
             }
             messages.append(message);
         }
@@ -249,7 +304,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     }
 
     if (allSignedOut) {
-        _tray->setIcon(Theme::instance()->folderOfflineIcon(true, contextMenuVisible()));
+        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Status::Offline, true, contextMenuVisible()));
         _tray->setToolTip(tr("Please sign in"));
         setStatusText(tr("Signed out"));
         return;
@@ -263,7 +318,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     // display the info of the least successful sync (eg. do not just display the result of the latest sync)
     QString trayMessage;
 
-    auto trayOverallStatusResult = FolderMan::trayOverallStatus(map.values());
+    auto trayOverallStatusResult = FolderMan::trayOverallStatus(map);
 
     const QIcon statusIcon = Theme::instance()->syncStateIcon(trayOverallStatusResult.overallStatus(), true, contextMenuVisible());
     _tray->setIcon(statusIcon);
@@ -288,10 +343,12 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     case SyncResult::Problem:
         if (trayOverallStatusResult.overallStatus().hasUnresolvedConflicts()) {
             setStatusText(tr("Unresolved %1 conflicts").arg(QString::number(trayOverallStatusResult.overallStatus().numNewConflictItems())));
+            break;
         } else if (trayOverallStatusResult.overallStatus().numBlacklistErrors() != 0) {
             setStatusText(tr("Ignored errors %1").arg(QString::number(trayOverallStatusResult.overallStatus().numBlacklistErrors())));
+            break;
         }
-        break;
+        [[fallthrough]];
     case SyncResult::Success: {
         QString lastSyncDoneString;
         // display only the time in case the last sync was today
@@ -305,6 +362,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     case SyncResult::Undefined:
         _tray->setToolTip(tr("There are no sync folders configured."));
         setStatusText(tr("No sync folders configured"));
+        break;
     default:
         setStatusText(FolderMan::instance()->trayTooltipStatusString(trayOverallStatusResult.overallStatus(), false));
     }
@@ -324,7 +382,7 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
 
     FolderMan *folderMan = FolderMan::instance();
     bool firstFolder = true;
-    const auto &map = folderMan->map();
+    const auto &map = folderMan->folders();
     bool singleSyncFolder = map.size() == 1 && Theme::instance()->singleSyncFolder();
     bool onePaused = false;
     bool allPaused = true;
@@ -346,8 +404,7 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
         }
 
         QAction *action = menu->addAction(tr("Open folder '%1'").arg(folder->shortGuiLocalPath()));
-        auto alias = folder->alias();
-        connect(action, &QAction::triggered, this, [this, alias] { this->slotFolderOpenAction(alias); });
+        connect(action, &QAction::triggered, this, [this, folder] { this->slotFolderOpenAction(folder); });
     }
 
     menu->addSeparator();
@@ -525,7 +582,7 @@ void ownCloudGui::setupContextMenu()
         || desktopSession.contains("plasma")
         || desktopSession.contains("kde");
     QObject *platformMenu = reinterpret_cast<QObject *>(_tray->contextMenu()->platformMenu());
-    if (isKde && platformMenu && platformMenu->metaObject()->className() == QLatin1String("QDBusPlatformMenu")) {
+    if (isKde && platformMenu && platformMenu->metaObject()->className() == QByteArrayLiteral("QDBusPlatformMenu")) {
         _workaroundManualVisibility = true;
         _workaroundNoAboutToShowUpdate = true;
     }
@@ -544,7 +601,7 @@ void ownCloudGui::setupContextMenu()
 
 
     connect(&_delayedTrayUpdateTimer, &QTimer::timeout, this, &ownCloudGui::updateContextMenu);
-    _delayedTrayUpdateTimer.setInterval(2 * 1000);
+    _delayedTrayUpdateTimer.setInterval(2s);
     _delayedTrayUpdateTimer.setSingleShot(true);
 
     connect(_contextMenu.data(), &QMenu::aboutToShow, this, &ownCloudGui::slotContextMenuAboutToShow);
@@ -611,7 +668,7 @@ void ownCloudGui::updateContextMenu()
         }
     }
 
-    for (auto *f : FolderMan::instance()->map()) {
+    for (auto *f : FolderMan::instance()->folders()) {
         if (f->syncPaused()) {
             atLeastOnePaused = true;
         } else {
@@ -659,9 +716,9 @@ void ownCloudGui::updateContextMenu()
     if (atLeastOnePaused) {
         QString text;
         if (accountList.count() > 1) {
-            text = tr("Unpause all synchronization");
+            text = tr("Restart all synchronization");
         } else {
-            text = tr("Unpause synchronization");
+            text = tr("Restart synchronization");
         }
         QAction *action = _contextMenu->addAction(text);
         connect(action, &QAction::triggered, this, &ownCloudGui::slotUnpauseAllFolders);
@@ -669,9 +726,9 @@ void ownCloudGui::updateContextMenu()
     if (atLeastOneNotPaused) {
         QString text;
         if (accountList.count() > 1) {
-            text = tr("Pause all synchronization");
+            text = tr("Stop all synchronization");
         } else {
-            text = tr("Pause synchronization");
+            text = tr("Stop synchronization");
         }
         QAction *action = _contextMenu->addAction(text);
         connect(action, &QAction::triggered, this, &ownCloudGui::slotPauseAllFolders);
@@ -744,9 +801,8 @@ void ownCloudGui::slotShowOptionalTrayMessage(const QString &title, const QStrin
 /*
  * open the folder with the given Alias
  */
-void ownCloudGui::slotFolderOpenAction(const QString &alias)
+void ownCloudGui::slotFolderOpenAction(Folder *f)
 {
-    Folder *f = FolderMan::instance()->folder(alias);
     if (f) {
         qCInfo(lcApplication) << "opening local url " << f->path();
         QUrl url = QUrl::fromLocalFile(f->path());
@@ -768,7 +824,7 @@ void ownCloudGui::setupActions()
 {
     _actionStatus = new QAction(tr("Unknown status"), this);
     _actionStatus->setEnabled(false);
-    _actionSettings = new QAction(tr("Settings..."), this);
+    _actionSettings = new QAction(tr("Show %1").arg(Theme::instance()->appNameGUI()), this);
     _actionNewAccountWizard = new QAction(tr("New account..."), this);
     _actionRecent = new QAction(tr("Details..."), this);
     _actionRecent->setEnabled(true);
@@ -789,11 +845,11 @@ void ownCloudGui::setupActions()
     connect(_actionLogout, &QAction::triggered, this, &ownCloudGui::slotLogout);
 
     if (_app->debugMode()) {
-        _actionCrash = new QAction("Crash now - Div by zero", this);
+        _actionCrash = new QAction(QStringLiteral("Crash now - Div by zero"), this);
         connect(_actionCrash, &QAction::triggered, _app, &Application::slotCrash);
-        _actionCrashEnforce = new QAction("Crash now - ENFORCE()", this);
+        _actionCrashEnforce = new QAction(QStringLiteral("Crash now - ENFORCE()"), this);
         connect(_actionCrashEnforce, &QAction::triggered, _app, &Application::slotCrashEnforce);
-        _actionCrashFatal = new QAction("Crash now - qFatal", this);
+        _actionCrashFatal = new QAction(QStringLiteral("Crash now - qFatal"), this);
         connect(_actionCrashFatal, &QAction::triggered, _app, &Application::slotCrashFatal);
     } else {
         _actionCrash = nullptr;
@@ -826,11 +882,8 @@ static bool shouldShowInRecentsMenu(const SyncFileItem &item)
         && item._instruction != CSYNC_INSTRUCTION_NONE;
 }
 
-
-void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &progress)
+void ownCloudGui::slotUpdateProgress(Folder *folder, const ProgressInfo &progress)
 {
-    Q_UNUSED(folder);
-
     if (progress.status() == ProgressInfo::Discovery) {
         if (!progress._currentDiscoveredRemoteFolder.isEmpty()) {
             _actionStatus->setText(tr("Checking for changes in remote '%1'")
@@ -840,7 +893,7 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
                                        .arg(progress._currentDiscoveredLocalFolder));
         }
     } else if (progress.status() == ProgressInfo::Done) {
-        QTimer::singleShot(2000, this, &ownCloudGui::slotComputeOverallSyncStatus);
+        QTimer::singleShot(2s, this, &ownCloudGui::slotComputeOverallSyncStatus);
     }
     if (progress.status() != ProgressInfo::Propagation) {
         return;
@@ -880,21 +933,18 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
         && shouldShowInRecentsMenu(progress._lastCompletedItem)) {
         if (Progress::isWarningKind(progress._lastCompletedItem._status)) {
             // display a warn icon if warnings happened.
-            _actionRecent->setIcon(Utility::getCoreIcon(QStringLiteral("warning")));
+            _actionRecent->setIcon(Resources::getCoreIcon(QStringLiteral("warning")));
         }
 
         QString kindStr = Progress::asResultString(progress._lastCompletedItem);
-        QString timeStr = QTime::currentTime().toString("hh:mm");
+        QString timeStr = QTime::currentTime().toString(QStringLiteral("hh:mm"));
         QString actionText = tr("%1 (%2, %3)").arg(progress._lastCompletedItem._file, kindStr, timeStr);
         QAction *action = new QAction(actionText, this);
-        Folder *f = FolderMan::instance()->folder(folder);
-        if (f) {
-            QString fullPath = f->path() + '/' + progress._lastCompletedItem._file;
-            if (QFile(fullPath).exists()) {
-                connect(action, &QAction::triggered, this, [this, fullPath] { this->slotOpenPath(fullPath); });
-            } else {
-                action->setEnabled(false);
-            }
+        QString fullPath = folder->path() + QLatin1Char('/') + progress._lastCompletedItem._file;
+        if (QFile(fullPath).exists()) {
+            connect(action, &QAction::triggered, this, [this, fullPath] { this->slotOpenPath(fullPath); });
+        } else {
+            action->setEnabled(false);
         }
         if (_recentItemsActions.length() > 5) {
             _recentItemsActions.takeFirst()->deleteLater();
@@ -912,7 +962,6 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo &
 void ownCloudGui::slotLogin()
 {
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
-        account->account()->resetRejectedCertificates();
         account->signIn();
     } else {
         for (const auto &a : AccountManager::instance()->accounts()) {
@@ -946,27 +995,125 @@ void ownCloudGui::slotPauseAllFolders()
 
 void ownCloudGui::runNewAccountWizard()
 {
-    if (!_wizard) {
-        _wizard = new OwncloudSetupWizard(settingsDialog());
-        connect(_wizard, &OwncloudSetupWizard::ownCloudWizardDone, _wizard, &OwncloudSetupWizard::deleteLater);
-        connect(_wizard, &OwncloudSetupWizard::ownCloudWizardDone, ocApp(), &Application::slotownCloudWizardDone);
+    if (_wizardController.isNull()) {
+        // passing the settings dialog as parent makes sure the wizard will be shown above it
+        // as the settingsDialog's lifetime spans across the entire application but the dialog will live much shorter,
+        // we have to clean it up manually when finished() is emitted
+        _wizardController = new Wizard::SetupWizardController(settingsDialog());
+
+        // while the wizard is shown, new syncs are disabled
         FolderMan::instance()->setSyncEnabled(false);
-        _wizard->startWizard();
+
+        connect(_wizardController, &Wizard::SetupWizardController::finished, ocApp(),
+            [this](AccountPtr newAccount, Wizard::SyncMode syncMode, const QVariantMap &dynamicRegistrationData) {
+                // note: while the wizard is shown, we disable the folder synchronization
+                // previously we could perform this just here, but now we have to postpone this depending on whether selective sync was chosen
+                // see also #9497
+
+                // when the dialog is closed before it has finished, there won't be a new account to set up
+                // the wizard controller signalizes this by passing a null pointer
+                if (!newAccount.isNull()) {
+                    // finally, call the slot that finalizes the setup
+                    auto accountStatePtr = ocApp()->addNewAccount(newAccount);
+
+                    // ensure we are connected and fetch the capabilities
+                    auto validator = new ConnectionValidator(accountStatePtr->account(), accountStatePtr->account().data());
+
+                    QObject::connect(validator, &ConnectionValidator::connectionResult, accountStatePtr.data(), [accountStatePtr, syncMode, dynamicRegistrationData](ConnectionValidator::Status status, const QStringList &errors) {
+                        if (OC_ENSURE(status == ConnectionValidator::Connected || status == ConnectionValidator::ServerVersionMismatch)) {
+                            // saving once after adding makes sure the account is stored in the config in a working state
+                            // this is needed to ensure a consistent state in the config file upon unexpected terminations of the client
+                            // (for instance, when running from a debugger and stopping the process from there)
+                            AccountManager::instance()->save(true);
+
+                            // only now, we can store the dynamic registration data in the keychain
+                            if (!dynamicRegistrationData.isEmpty()) {
+                                OAuth::saveDynamicRegistrationDataForAccount(accountStatePtr->account(), dynamicRegistrationData);
+                            }
+
+                            // the account is now ready, emulate a normal account loading and emit that the credentials are ready
+                            Q_EMIT accountStatePtr->account()->credentialsFetched(accountStatePtr->account()->credentials());
+
+                            switch (syncMode) {
+                            case Wizard::SyncMode::SyncEverything:
+                            case Wizard::SyncMode::UseVfs: {
+                                bool useVfs = syncMode == Wizard::SyncMode::UseVfs;
+                                setUpInitialSyncFolder(accountStatePtr, useVfs);
+
+                                break;
+                            }
+                            case Wizard::SyncMode::ConfigureUsingFolderWizard: {
+                                Q_ASSERT(!accountStatePtr->account()->hasDefaultSyncRoot());
+
+                                auto *folderWizard = new FolderWizard(accountStatePtr, ocApp()->gui()->settingsDialog());
+                                folderWizard->resize(ocApp()->gui()->settingsDialog()->sizeHintForChild());
+                                folderWizard->setAttribute(Qt::WA_DeleteOnClose);
+
+                                // TODO: duplication of AccountSettings
+                                // adapted from AccountSettings::slotFolderWizardAccepted()
+                                connect(folderWizard, &QDialog::accepted, [accountStatePtr, folderWizard]() {
+                                    FolderMan *folderMan = FolderMan::instance();
+
+                                    qCInfo(lcApplication) << "Folder wizard completed";
+                                    const auto config = folderWizard->result();
+
+                                    auto folder = folderMan->addFolderFromFolderWizardResult(accountStatePtr, config);
+
+                                    if (!config.selectiveSyncBlackList.isEmpty() && OC_ENSURE(folder && !config.useVirtualFiles)) {
+                                        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, config.selectiveSyncBlackList);
+
+                                        // The user already accepted the selective sync dialog. everything is in the white list
+                                        folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, { QLatin1String("/") });
+                                    }
+
+                                    folderMan->setSyncEnabled(true);
+                                    folderMan->scheduleAllFolders();
+                                });
+
+                                connect(folderWizard, &QDialog::rejected, []() {
+                                    qCInfo(lcApplication) << "Folder wizard cancelled";
+                                    FolderMan::instance()->setSyncEnabled(true);
+                                });
+
+                                folderWizard->open();
+                                ocApp()->gui()->raiseDialog(folderWizard);
+
+                                break;
+                            }
+                            default:
+                                Q_UNREACHABLE();
+                            }
+                        }
+                    });
+
+
+                    validator->checkServer();
+                } else {
+                    FolderMan::instance()->setSyncEnabled(true);
+                }
+
+                // make sure the wizard is cleaned up eventually
+                _wizardController->deleteLater();
+            });
+
+        // all we have to do is show the dialog...
+        _wizardController->window()->open();
+        // ... and bring it to the front
+        raiseDialog(_wizardController->window());
     }
-    raiseDialog(settingsDialog());
 }
 
 void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
 {
-    QList<AccountState *> accounts;
+    QList<AccountStatePtr> accounts;
     if (auto account = qvariant_cast<AccountStatePtr>(sender()->property(propertyAccountC))) {
-        accounts.append(account.data());
+        accounts.append(account);
     } else {
         for (const auto &a : AccountManager::instance()->accounts()) {
-            accounts.append(a.data());
+            accounts.append(a);
         }
     }
-    for (auto *f : FolderMan::instance()->map()) {
+    for (auto *f : FolderMan::instance()->folders()) {
         if (accounts.contains(f->accountState())) {
             f->setSyncPaused(pause);
             if (pause) {
@@ -1032,15 +1179,20 @@ void ownCloudGui::slotHelp()
 void ownCloudGui::raiseDialog(QWidget *raiseWidget)
 {
     auto window = ocApp()->gui()->settingsDialog();
+    auto *dialog = qobject_cast<QDialog *>(raiseWidget);
     OC_ASSERT(window);
-    OC_ASSERT_X(!qobject_cast<QDialog *>(raiseWidget) || raiseWidget->parentWidget() == window, "raiseDialog should only be called with modal dialogs");
+    OC_ASSERT_X(!dialog || raiseWidget->parentWidget() == window, "raiseDialog should only be called with modal dialogs");
     if (!window) {
         return;
     }
     window->showNormal();
     window->raise();
-    raiseWidget->showNormal();
-    raiseWidget->raise();
+    if (dialog && !dialog->isVisible()) {
+        dialog->open();
+    } else {
+        raiseWidget->showNormal();
+        raiseWidget->raise();
+    }
     window->activateWindow();
     raiseWidget->activateWindow();
 
@@ -1071,47 +1223,53 @@ void ownCloudGui::slotShowShareDialog(const QString &sharePath, const QString &l
         qCWarning(lcApplication) << "Could not open share dialog for" << localPath << "no responsible folder found";
         return;
     }
-
-    const auto accountState = folder->accountState();
-
-    SyncJournalFileRecord fileRecord;
-
-    bool resharingAllowed = true; // lets assume the good
-    if (folder->journalDb()->getFileRecord(file, &fileRecord) && fileRecord.isValid()) {
-        // check the permission: Is resharing allowed?
-        if (!fileRecord._remotePerm.isNull() && !fileRecord._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
-            resharingAllowed = false;
-        }
-    }
-
-    // As a first approximation, set the set of permissions that can be granted
-    // either to everything (resharing allowed) or nothing (no resharing).
-    //
-    // The correct value will be found with a propfind from ShareDialog.
-    // (we want to show the dialog directly, not wait for the propfind first)
-    SharePermissions maxSharingPermissions =
-        SharePermissionRead
-        | SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete
-        | SharePermissionShare;
-    if (!resharingAllowed) {
-        maxSharingPermissions = SharePermission(0);
-    }
-
-
-    ShareDialog *w = nullptr;
-    if (_shareDialogs.contains(localPath) && _shareDialogs[localPath]) {
-        qCInfo(lcApplication) << "Raising share dialog" << sharePath << localPath;
-        w = _shareDialogs[localPath];
+    if (folder->accountState()->account()->capabilities().filesSharing().sharing_roles) {
+        fetchPrivateLinkUrl(folder->accountState()->account(), folder->webDavUrl(), sharePath, this, [](const QUrl &url) {
+            const auto queryUrl = Utility::concatUrlPath(url, QString(), {{QStringLiteral("details"), QStringLiteral("sharing")}});
+            Utility::openBrowser(queryUrl, nullptr);
+        });
     } else {
-        qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
-        w = new ShareDialog(accountState, sharePath, localPath, maxSharingPermissions, startPage, settingsDialog());
-        w->setAttribute(Qt::WA_DeleteOnClose, true);
+        const auto accountState = folder->accountState();
 
-        _shareDialogs[localPath] = w;
-        connect(w, &QObject::destroyed, this, &ownCloudGui::slotRemoveDestroyedShareDialogs);
+        SyncJournalFileRecord fileRecord;
+
+        bool resharingAllowed = true; // lets assume the good
+        if (folder->journalDb()->getFileRecord(file, &fileRecord) && fileRecord.isValid()) {
+            // check the permission: Is resharing allowed?
+            if (!fileRecord._remotePerm.isNull() && !fileRecord._remotePerm.hasPermission(RemotePermissions::CanReshare)) {
+                resharingAllowed = false;
+            }
+        }
+
+        // As a first approximation, set the set of permissions that can be granted
+        // either to everything (resharing allowed) or nothing (no resharing).
+        //
+        // The correct value will be found with a propfind from ShareDialog.
+        // (we want to show the dialog directly, not wait for the propfind first)
+        SharePermissions maxSharingPermissions =
+            SharePermissionRead
+            | SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete
+            | SharePermissionShare;
+        if (!resharingAllowed) {
+            maxSharingPermissions = SharePermission(0);
+        }
+
+
+        ShareDialog *w = nullptr;
+        if (_shareDialogs.contains(localPath) && _shareDialogs[localPath]) {
+            qCInfo(lcApplication) << "Raising share dialog" << sharePath << localPath;
+            w = _shareDialogs[localPath];
+        } else {
+            qCInfo(lcApplication) << "Opening share dialog" << sharePath << localPath << maxSharingPermissions;
+            w = new ShareDialog(accountState, folder->webDavUrl(), sharePath, localPath, maxSharingPermissions, startPage, settingsDialog());
+            w->setAttribute(Qt::WA_DeleteOnClose, true);
+
+            _shareDialogs[localPath] = w;
+            connect(w, &QObject::destroyed, this, &ownCloudGui::slotRemoveDestroyedShareDialogs);
+        }
+        w->open();
+        raiseDialog(w);
     }
-    w->open();
-    raiseDialog(w);
 }
 
 void ownCloudGui::slotRemoveDestroyedShareDialogs()
